@@ -1,6 +1,6 @@
 # encoding:utf-8
 #--
-# Copyright (C) 2006-2013 Bob Aman
+# Copyright (C) Bob Aman
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -21,23 +21,39 @@ require "idn"
 module Addressable
   module IDNA
     def self.punycode_encode(value)
-      IDN::Punycode.encode(value)
+      IDN::Punycode.encode(value.to_s)
     end
 
      def self.punycode_decode(value)
-       IDN::Punycode.decode(value)
+       IDN::Punycode.decode(value.to_s)
      end
 
     def self.unicode_normalize_kc(value)
-      IDN::Stringprep.nfkc_normalize(value)
+      IDN::Stringprep.nfkc_normalize(value.to_s)
     end
 
     def self.to_ascii(value)
-      IDN::Idna.toASCII(value)
+      value.to_s.split('.', -1).map do |segment|
+        if segment.size > 0 && segment.size < 64
+          IDN::Idna.toASCII(segment, IDN::Idna::ALLOW_UNASSIGNED)
+        elsif segment.size >= 64
+          segment
+        else
+          ''
+        end
+      end.join('.')
     end
 
     def self.to_unicode(value)
-      IDN::Idna.toUnicode(value)
+      value.to_s.split('.', -1).map do |segment|
+        if segment.size > 0 && segment.size < 64
+          IDN::Idna.toUnicode(segment, IDN::Idna::ALLOW_UNASSIGNED)
+        elsif segment.size >= 64
+          segment
+        else
+          ''
+        end
+      end.join('.')
     end
   end
 end
