@@ -1,6 +1,6 @@
 # encoding:utf-8
 #--
-# Copyright (C) 2006-2013 Bob Aman
+# Copyright (C) Bob Aman
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -64,6 +64,7 @@ module Addressable
     # Converts from a Unicode internationalized domain name to an ASCII
     # domain name as described in RFC 3490.
     def self.to_ascii(input)
+      input = input.to_s unless input.is_a?(String)
       input = input.dup
       if input.respond_to?(:force_encoding)
         input.force_encoding(Encoding::ASCII_8BIT)
@@ -89,10 +90,16 @@ module Addressable
     # Converts from an ASCII domain name to a Unicode internationalized
     # domain name as described in RFC 3490.
     def self.to_unicode(input)
+      input = input.to_s unless input.is_a?(String)
       parts = input.split('.')
       parts.map! do |part|
-        if part =~ /^#{ACE_PREFIX}/
-          punycode_decode(part[/^#{ACE_PREFIX}(.+)/, 1])
+        if part =~ /^#{ACE_PREFIX}(.+)/
+          begin
+            punycode_decode(part[/^#{ACE_PREFIX}(.+)/, 1])
+          rescue Addressable::IDNA::PunycodeBadInput
+            # toUnicode is explicitly defined as never-fails by the spec
+            part
+          end
         else
           part
         end
@@ -121,6 +128,7 @@ module Addressable
     #   The input string.
     # @return [String] The downcased result.
     def self.unicode_downcase(input)
+      input = input.to_s unless input.is_a?(String)
       unpacked = input.unpack("U*")
       unpacked.map! { |codepoint| lookup_unicode_lowercase(codepoint) }
       return unpacked.pack("U*")
@@ -172,7 +180,6 @@ module Addressable
 
       p = []
       ucs4_to_utf8 = lambda do |ch|
-        # For some reason, rcov likes to drop BUS errors here.
         if ch < 128
           p << ch
         elsif ch < 2048
@@ -376,6 +383,7 @@ module Addressable
     class PunycodeOverflow < StandardError; end
 
     def self.punycode_encode(unicode)
+      unicode = unicode.to_s unless unicode.is_a?(String)
       input = unicode.unpack("U*")
       output = [0] * (ACE_MAX_LENGTH + 1)
       input_length = input.size
@@ -484,7 +492,7 @@ module Addressable
       outlen.times do |j|
         c = output[j]
         unless c >= 0 && c <= 127
-          raise Exception, "Invalid output char."
+          raise StandardError, "Invalid output char."
         end
         unless PUNYCODE_PRINT_ASCII[c]
           raise PunycodeBadInput, "Input is invalid."
